varying vec3 pos, normal;

uniform float rough, specularity, ior;
uniform float dif_sdr, spec_sdr;

#define DSDR_LAMBERT	0.0
#define DSDR_OREN_NAYAR	1.0

#define SSDR_PHONG		0.0
#define SSDR_BLINN		1.0
#define SSDR_COOK_TORR	2.0

#define EQ(a, b)	(abs((a) - (b)) < 0.5)

float lambert(vec3 norm, vec3 ldir);
float oren_nayar(vec3 view, vec3 norm, vec3 ldir, float rough);

float phong(vec3 view, vec3 norm, vec3 ldir, float spow);
float blinn(vec3 norm, vec3 hvec, float spow);
float cook_torrance(vec3 view, vec3 norm, vec3 ldir, vec3 hvec, float rough, float ior);

void main()
{
	vec4 color = vec4(1.0, 1.0, 1.0, 1.0);
	float idiff, ispec;

	vec3 view = -normalize(pos);
	vec3 norm = normalize(normal);
	vec3 ldir = normalize(gl_LightSource[0].position.xyz - pos);

	if(EQ(dif_sdr, DSDR_LAMBERT)) {
		idiff = lambert(norm, ldir);
	} else if(EQ(dif_sdr, DSDR_OREN_NAYAR)) {
		idiff = oren_nayar(view, norm, ldir, rough);
	} else {
		idiff = 0.0;
	}

	vec4 mcolor = gl_FrontMaterial.diffuse;

	if(EQ(spec_sdr, SSDR_PHONG)) {
		ispec = phong(view, norm, ldir, gl_FrontMaterial.shininess);
		color = mcolor * idiff + vec4(specularity, specularity, specularity, 1) * ispec;
	} else if(EQ(spec_sdr, SSDR_BLINN)) {
		ispec = blinn(norm, gl_LightSource[0].halfVector.xyz, gl_FrontMaterial.shininess);
		color = mcolor * idiff + vec4(specularity, specularity, specularity, 1) * ispec;
	} else if(EQ(spec_sdr, SSDR_COOK_TORR)) {
		ispec = cook_torrance(view, norm, ldir, gl_LightSource[0].halfVector.xyz, rough, ior);
		color = specularity * ispec * mcolor + (1.0 - specularity) * idiff * mcolor;
	} else {
		color = vec4(1, 0, 0, 1);
	}
	color.a = 1.0;

	gl_FragColor = color;
}
